package fileChooser;

import graphLib.*;
import java.awt.geom.Point2D;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Read and save for .net files
 * @author tadaki
 */
public class DotNetData extends NetworkFile {
    
    private HashMap<Vertex,String> colorMap=null;
    private enum ReadMode {

        VertexHeaderMode, LinkHeaderMode, VertexMode, LinkMode;
    }
    
    @Override
    public boolean output(Network network, String outputFile, String path)
            throws IOException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public boolean output(Network network, String outputFile)
            throws IOException {
        return output(network, FileUtil.openOutputStream(outputFile));
    }

    @Override
    public boolean output(Network network, OutputStreamWriter swriter)
            throws IOException {
        try (BufferedWriter writer = new BufferedWriter(swriter)) {
            outputComment(writer);
            saveVertexes(network, writer);
            saveLinks(network, writer);
            writer.flush();
            writer.close();
        }
        return true;
    }

    protected void outputComment(BufferedWriter writer) throws IOException {
        String[] localComments = getComments();
        if(localComments==null || localComments.length==0)return;
        for(int i=0;i<localComments.length;i++){
            writer.write("%");
            writer.write(localComments[i]);
            writer.newLine();
        }
    }

    private boolean saveVertexes(Network network, BufferedWriter writer)
            throws IOException {
        StringBuilder sb = new StringBuilder();
        int n = network.getSize();
        sb.append("*Vertices ").append(n);
        writer.write(sb.toString());
        writer.newLine();
        for (int i = 0; i < n; i++) {
            sb = new StringBuilder();
            sb.append(i + 1);
            Vertex v = network.getVertexes().get(i);
            sb.append(" \"").append(v.getName()).append("\"");
            Point2D.Double p = v.getPoint();
            if (p != null) {
                sb.append(" ").append(p.x).append(" ").append(p.y).append(" 0.0");
            }
            if(colorMap!=null){
                String color = colorMap.get(v);
                sb.append(" ic ").append(color);
            }
            writer.write(sb.toString());
            writer.newLine();
        }
        return true;
    }

    private boolean saveLinks(Network network, BufferedWriter writer)
            throws IOException {
        if (network.isDirected()) {
            writer.write("*Arcs");
        } else {
            writer.write("*Edges");
        }
        writer.newLine();
        int n = network.getSize();
        List<Vertex> vList = network.getVertexes();
        for (int i = 0; i < n; i++) {
            Vertex v = vList.get(i);
            List<Arc> aList = network.getArcs(v);
            if (aList != null) {
                for (Arc a : aList) {
                    StringBuilder sb = new StringBuilder();
                    Vertex w = network.getTerminal(a, v);
                    int j = vList.indexOf(w);
                    if (network.isDirected()) {
                        sb.append(i + 1).append(" ").append(j + 1);
                        sb.append(" ").append(a.getWeight());
                        writer.write(sb.toString());
                        writer.newLine();
                    } else {
                        if (j > i) {
                            sb.append(i + 1).append(" ").append(j + 1);
                            sb.append(" ").append(a.getWeight());
                            writer.write(sb.toString());
                            writer.newLine();
                        }
                    }
                }
            }
        }
        return true;
    }

    @Override
    public Network input(String filename)
            throws IOException {
        return input(new File(filename));
    }

    @Override
    public Network input(File file)
            throws IOException {
        return input(FileUtil.openInputStream(file));
    }

    @Override
    public Network input(InputStreamReader sreader) throws IOException {
        BufferedReader reader = new BufferedReader(sreader);
        String line;
        Network network = new Network("network");
        network.setDirected(true);
        ReadMode mode = ReadMode.VertexHeaderMode;
        while ((line = reader.readLine()) != null) {
            String l = line.trim();
            String s[] = l.split("\\s+");
            if (l.startsWith("*Vertices")) {
                mode = ReadMode.VertexHeaderMode;
            } else {
                if (l.startsWith("*Edges")) {
                    mode = ReadMode.LinkHeaderMode;
                    network.setDirected(false);
                }
                if (l.startsWith("*Arcs")) {
                    mode = ReadMode.LinkHeaderMode;
                }
            }
            if (s.length > 0) {
                switch (mode) {
                    case VertexHeaderMode:
                        mode = ReadMode.VertexMode;
                        break;
                    case LinkHeaderMode:
                        mode = ReadMode.LinkMode;
                        break;
                    case VertexMode:
                        readVertex(network, s);
                        break;
                    case LinkMode:
                        readLink(network, s);
                        break;
                    default:
                        break;
                }
            }
        }
        return network;
    }

    private void readVertex(Network network, String s[]) {
        String label = s[1].trim();
        int l = label.length();
        label = label.substring(1, l - 1);
        Vertex v = network.addVertex(label);
        if (s.length > 3) {
            try {
                double x = Double.valueOf(s[2]);
                double y = Double.valueOf(s[3]);
                v.setPoint(x, y);
            } catch (NumberFormatException ex) {
                Logger.getLogger(DotNetData.class.getName()).
                        log(Level.SEVERE,
                        "Invalid Number Format {0}", s[2] + " " + s[3]);
            }
        }
        if(s.length>6 && s[5].equals("ic")){
            if(colorMap==null){
                colorMap = new HashMap<>();
                String color = s[6];
                colorMap.put(v, color);
            }
        }
    }

    private void readLink(Network network, String s[]) {
        int from = 0;
        int to = 0;
        try {
            from = Integer.valueOf(s[0]);
            to = Integer.valueOf(s[1]);
        } catch (NumberFormatException ex) {
            Logger.getLogger(DotNetData.class.getName()).
                    log(Level.SEVERE,
                    "Invalid Number Format {0}", s[0] + " " + s[1]);
        }
        List<Vertex> vList = network.getVertexes();
        Vertex f = vList.get(from - 1);
        Vertex t = vList.get(to - 1);
        Arc a = network.addArc(f, t, "");
        if (s.length > 2) {
            try {
                double w = Double.valueOf(s[2]);
                a.setWeight(w);
            } catch (NumberFormatException ex) {
                Logger.getLogger(DotNetData.class.getName()).
                        log(Level.SEVERE,
                        "Invalid Number Format {0}", s[2]);
            }
        }
    }

    public HashMap<Vertex, String> getColorMap() {
        return colorMap;
    }

    public void setColorMap(HashMap<Vertex, String> colorMap) {
        this.colorMap = colorMap;
    }
    
    
}
