package graphLib;

import java.util.HashMap;
import java.util.List;

/**
 *
 * @author tadaki
 */
public abstract class GraphBase {

    protected HashMap<Arc, Vertex> a2vHead = null;
    protected HashMap<Arc, Vertex> a2vTail = null;
    protected List<Arc> arcs = null;
    /** 有向グラフであるか */
    protected boolean directed = true;
    protected String name = null;
    protected HashMap<Vertex, List<Arc>> v2a = null;
    protected List<Vertex> vertexes = null;

    /** コンストラクタ */
    public GraphBase() {
        name = "Graph";
    }

    /**
     * 名前を指定したコンストラクタ
     * @param name
     */
    public GraphBase(String name) {
        this.name = name;
    }

    /**
     * 弧の追加
     * @param fromName 弧の始点の頂点の名前
     * @param toName   弧の終点の頂点の名前
     * @param name     弧の名前
     * @return         生成した弧
     */
    public Arc addArc(String fromName, String toName, String name) {
        if (vertexes.isEmpty()) {
            return null;
        }
        Vertex from = getVertex(fromName);
        Vertex to = getVertex(toName);
        return addArc(from, to, name);
    }

    /**
     * 弧の追加
     * @param fromName 弧の始点の頂点
     * @param toName   弧の終点の頂点
     * @param name     弧の名前
     * @return         生成した弧
     */
    public Arc addArc(Vertex from, Vertex to, String name) {
        if (from == null || to == null) {
            return null;
        }
        Arc a = new Arc(name);
        arcs.add(a);
        a2vHead.put(a, from);
        a2vTail.put(a, to);
        if (!v2a.containsKey(from)) {
            v2a.put(from, Utils.createArcList());
        }
        List<Arc> aList = v2a.get(from);
        aList.add(a);
        if (!directed) {
            //無向グラフの場合
            if (!v2a.containsKey(to)) {
                v2a.put(to, Utils.createArcList());
            }
            aList = v2a.get(to);
            aList.add(a);
        }
        return a;
    }

    /**
     * 頂点の追加
     * @param name 頂点の名前
     * @return     生成した頂点
     */
    public Vertex addVertex(String name) {
        Vertex v = new Vertex(name);
        this.addVertex(v);
        return v;
    }

    /**
     * 頂点の追加
     * @param v
     */
    public void addVertex(Vertex v) {
        vertexes.add(v);
    }

    /**
     * 頂点の名前を指定して弧を取得
     * @param name 頂点を指定する名前
     * @param i    頂点の持つ弧を指定
     * @return
     */
    public Arc getArc(String name, int i) {
        Vertex v = getVertex(name);
        if (v == null) {
            return null;
        }
        return v2a.get(v).get(i);
    }

    /**
     * 始点と終点を指定して弧を取得
     * @param from 始点
     * @param to   終点
     * @return
     */
    public Arc getArc(Vertex from, Vertex to) {
        Arc a = null;
        if (from == null) {
            return null;
        }
        for (Arc b : v2a.get(from)) {
            if (a2vTail.get(b).equals(to)) {
                a = b;
            }
            if (!directed) {
                if (a2vHead.get(b).equals(to)) {
                    a = b;
                }
            }
        }
        return a;
    }

    /**
     * 弧を取得
     * @param i
     * @return
     */
    public Arc getArc(int i) {
        return arcs.get(i);
    }

    /**
     * 弧の一覧を取得
     * @return
     */
    public List<Arc> getArcs() {
        return arcs;
    }

    /**
     * 頂点を指定して弧を取得
     * @param v
     * @return
     */
    public List<Arc> getArcs(Vertex v) {
        return v2a.get(v);
    }

    /**
     * 頂点の名前を指定して、次数を取得
     * @param name 頂点を指定する名前
     * @return
     */
    public int getDegree(String name) {
        return getDegree(getVertex(name));
    }

    /**
     * 頂点の次数を取得
     * @param v
     * @return
     */
    public int getDegree(Vertex v) {
        List<Arc> arcsOfv = v2a.get(v);
        if(arcsOfv==null)return 0;
        return arcsOfv.size();
    }

    /**
     * 弧の始点を取得
     * @param a
     * @return
     */
    public Vertex getHead(Arc a) {
        return a2vHead.get(a);
    }

    /**
     * 弧の終点を取得
     * @param a
     * @return
     */
    public Vertex getTail(Arc a) {
        return a2vTail.get(a);
    }

    /**
     * 弧の指定した頂点の反対側の頂点を取得
     * @param a
     * @param v
     * @return
     */
    public Vertex getTerminal(Arc a, Vertex v) {
        Vertex t = null;
        Vertex head = a2vHead.get(a);
        Vertex tail = a2vTail.get(a);
        if (head.equals(v)) {
            t = tail;
        }
        if (tail.equals(v)) {
            t = head;
        }
        return t;
    }

    /**
     * グラフの名前を取得
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * 弧の数を返す
     * @return graphに含まれる弧の数
     */
    public int getNumArcs() {
        return arcs.size();
    }

    /**
     * 頂点数を返す
     * @return graphに含まれる頂点数
     */
    public int getSize() {
        return vertexes.size();
    }

    /**
     * 指定した名前を持つ頂点を取得
     * @param name
     * @return
     */
    public Vertex getVertex(String name) {
        Vertex vv = null;
        if (vertexes.isEmpty()) {
            return null;
        }
        for (Vertex v : vertexes) {
            if (v.toString().equals(name)) {
                vv = v;
            }
        }
        return vv;
    }

    /**
     * 頂点の一覧を取得
     * @param name
     * @return
     */
    public List<Vertex> getVertexes() {
        return vertexes;
    }

    /**
     * グラフの有向・無向を調べる
     * @return 有向の時に真
     */
    public boolean isDirected() {
        return directed;
    }

    /**
     * グラフの有向・無向を設定する
     * @param b 有向の時に真
     */
    public void setDirected(boolean b) {
        directed = b;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String toString() {
        return name;
    }
}
