package utils;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 *
 * @author tadaki
 */
public class BinaryHeap<T> {

    /** データを保持するリスト */
    private List<T> list;
    /** 要素を比較する方法 */
    private Comparator<T> comparator = null;
    /** 要素数 */
    private int n;

    /**
     * コンストラクタ：比較方法を指定する場合
     * 比較方法を指定しない場合は、
     *     要素はインターフェイスComparableを実装していること
     * @param comparator
     */
    public BinaryHeap(Comparator<T> comparator) {
        this();
        this.comparator = comparator;
    }

    public BinaryHeap() {
        list = Collections.synchronizedList(new ArrayList<T>());
        list.add(null);
        n = 0;
    }

    /**
     * 新しい要素を追加する
     * @param t
     * @return
     */
    public boolean add(T t) {
        boolean b = list.add(t);
        if (b) {
            n++;
            shiftUp(n);
        }
        return b;
    }

    /**
     * 最小の要素を得る：削除しない
     * @return
     */
    public T peek() {
        if (n == 0) {
            return null;
        }
        return list.get(1);
    }

    /**
     * 最小の要素を取り出し、削除する
     * @return
     */
    public T poll() {
        T t = null;
        if (n == 0) {
            return t;
        }
        if (n == 1) {//残りの要素が一つ
            t = list.remove(n);
            n--;
        } else {
            T x = list.remove(n);
            t = list.get(1);
            n--;
            list.set(1, x);
            shiftDown(1);
        }
        return t;
    }

    /**
     * 特定の要素の値を小さくした場合の再配置
     * @param t
     */
    public void reduceValue(T t) {
        int k = list.indexOf(t);
        shiftUp(k);
    }

    /**
     * 特定の要素の値を大きくした場合の再配置
     * @param t
     */
    public void raiseValue(T t){
        int k = list.indexOf(t);
        shiftDown(k);
    }

    /**
     * リストの取得
     * @return
     */
    public List<T> getList() {
        return list;
    }

    public boolean isEmpty() {
        return (n == 0);
    }

    public boolean contains(T t) {
        return list.contains(t);
    }

    /********************************************************************/
    /**
     * あるk にあるobjectを上位の適切な位置に置く
     * @param k
     */
    private void shiftUp(int k) {
        if (k > 1 && isLess(k, (int) (k / 2))) {
            int j = (int) (k / 2);
            swap(k, j);
            shiftUp(j);
        }
    }

    /**
     * あるk にあるobjectを下位の適切な位置に置く
     * @param k
     */
    private void shiftDown(int k) {
        if (2 * k <= n) {
            int j = 2 * k;
            if (j < n && isLess(j + 1, j)) {
                j++;
            }
            if (isLess(k, j)) {
                return;
            }
            swap(k, j);
            shiftDown(j);
        }
    }

    @SuppressWarnings("unchecked")
    private boolean isLess(int i, int j) {
        int a;
        T x = list.get(i);
        T y = list.get(j);
        if (comparator == null
                && x instanceof Comparable && y instanceof Comparable) {
            a = ((Comparable) x).compareTo((Comparable) y);
        } else {
            a = comparator.compare(x, y);
        }
        return (a < 0);
    }

    private void swap(int i, int j) {
        T o = list.get(i);
        list.set(i, list.get(j));
        list.set(j, o);
    }

    /********************************************************************/
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        /** サンプルとなるデータオブジェクト */
        class Data {

            int label;
            double value;

            public Data(int label, double value) {
                this.label = label;
                this.value = value;
            }
        }
        /** サンプルとなるデータの比較方法 */
        class CompareData implements Comparator<Data> {

            @Override
            public int compare(Data o1, Data o2) {
                int a = 0;
                if (o1.value > o2.value) {
                    a = 1;
                }
                if (o1.value < o2.value) {
                    a = -1;
                }
                return a;
            }
        }
        BinaryHeap<Data> h = new BinaryHeap<>(new CompareData());
        /** データ追加 */
        int n = 20;
        for (int i = 0; i < n; i++) {
            Data d = new Data(i + 1, Math.random());
            h.add(d);
        }
        /** 結果取得 */
        List<Data> list = h.getList();

        /** 最小値を削除 */
        for (int i = 0; i < 2; i++) {
            Data d = h.poll();
            n--;
        }
        /** 出力 */
        NumberFormat format = NumberFormat.getInstance();
        format.setMaximumFractionDigits(4);

        int l = (int) (Math.log((double) n) / Math.log(2.) + 0.1);
        for (int i = 0; i <= l; i++) {
            int m = (int) (Math.pow(2., (double) i) + 0.1);
            System.out.println();
            for (int j = 0; j < m; j++) {
                int k = m + j;
                if (k != 0 && k <= n) {
                    System.out.print("(");
                    System.out.print(list.get(k).label);
                    System.out.print(",");
                    System.out.print(format.format(list.get(k).value));
                    System.out.print(") ");
                }
            }
        }
        System.out.println();
    }
}
